@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2018 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2018 - 2026 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -73,20 +73,32 @@ static void _chat_log_chat(const char* const login, const char* const other, con
                            chat_log_direction_t direction, GDateTime* timestamp, const char* const resourcepart);
 static void _groupchat_log_chat(const gchar* const login, const gchar* const room, const gchar* const nick,
                                 const gchar* const msg);
+
 void
-chat_log_init(void)
+_chatlog_close(void)
+{
+    g_hash_table_destroy(logs);
+    g_hash_table_destroy(groupchat_logs);
+}
+
+void
+chatlog_init(void)
 {
     log_info("Initialising chat logs");
+
+    prof_add_shutdown_routine(_chatlog_close);
+
     logs = g_hash_table_new_full(g_str_hash, (GEqualFunc)_key_equals, free,
                                  (GDestroyNotify)_free_chat_log);
+    groupchat_logs = g_hash_table_new_full(g_str_hash, (GEqualFunc)_key_equals, free,
+                                           (GDestroyNotify)_free_chat_log);
 }
 
 void
 chat_log_msg_out(const char* const barejid, const char* const msg, const char* const resource)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        auto_char char* mybarejid = connection_get_barejid();
-        _chat_log_chat(mybarejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
+        _chat_log_chat(connection_get_barejid(), barejid, msg, PROF_OUT_LOG, NULL, resource);
     }
 }
 
@@ -94,7 +106,7 @@ void
 chat_log_otr_msg_out(const char* const barejid, const char* const msg, const char* const resource)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        auto_char char* mybarejid = connection_get_barejid();
+        const char* mybarejid = connection_get_barejid();
         auto_gchar gchar* pref_otr_log = prefs_get_string(PREF_OTR_LOG);
         if (strcmp(pref_otr_log, "on") == 0) {
             _chat_log_chat(mybarejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
@@ -108,7 +120,7 @@ void
 chat_log_pgp_msg_out(const char* const barejid, const char* const msg, const char* const resource)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        auto_char char* mybarejid = connection_get_barejid();
+        const char* mybarejid = connection_get_barejid();
         auto_gchar gchar* pref_pgp_log = prefs_get_string(PREF_PGP_LOG);
         if (strcmp(pref_pgp_log, "on") == 0) {
             _chat_log_chat(mybarejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
@@ -122,7 +134,7 @@ void
 chat_log_omemo_msg_out(const char* const barejid, const char* const msg, const char* const resource)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        auto_char char* mybarejid = connection_get_barejid();
+        const char* mybarejid = connection_get_barejid();
         auto_gchar gchar* pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
         if (strcmp(pref_omemo_log, "on") == 0) {
             _chat_log_chat(mybarejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
@@ -136,7 +148,7 @@ void
 chat_log_otr_msg_in(ProfMessage* message)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        auto_char char* mybarejid = connection_get_barejid();
+        const char* mybarejid = connection_get_barejid();
         auto_gchar gchar* pref_otr_log = prefs_get_string(PREF_OTR_LOG);
         if (message->enc == PROF_MSG_ENC_NONE || (strcmp(pref_otr_log, "on") == 0)) {
             if (message->type == PROF_MSG_TYPE_MUCPM) {
@@ -158,7 +170,7 @@ void
 chat_log_pgp_msg_in(ProfMessage* message)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        auto_char char* mybarejid = connection_get_barejid();
+        const char* mybarejid = connection_get_barejid();
         auto_gchar gchar* pref_pgp_log = prefs_get_string(PREF_PGP_LOG);
         if (strcmp(pref_pgp_log, "on") == 0) {
             if (message->type == PROF_MSG_TYPE_MUCPM) {
@@ -180,7 +192,7 @@ void
 chat_log_omemo_msg_in(ProfMessage* message)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        auto_char char* mybarejid = connection_get_barejid();
+        const char* mybarejid = connection_get_barejid();
         auto_gchar gchar* pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
         if (strcmp(pref_omemo_log, "on") == 0) {
             if (message->type == PROF_MSG_TYPE_MUCPM) {
@@ -202,7 +214,7 @@ void
 chat_log_msg_in(ProfMessage* message)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        auto_char char* mybarejid = connection_get_barejid();
+        const char* mybarejid = connection_get_barejid();
 
         if (message->type == PROF_MSG_TYPE_MUCPM) {
             _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
@@ -296,21 +308,12 @@ _chat_log_chat(const char* const login, const char* const other, const char* msg
     g_date_time_unref(timestamp);
 }
 
-void
-groupchat_log_init(void)
-{
-    log_info("Initialising groupchat logs");
-    groupchat_logs = g_hash_table_new_full(g_str_hash, (GEqualFunc)_key_equals, free,
-                                           (GDestroyNotify)_free_chat_log);
-}
-
 void
 groupchat_log_msg_out(const gchar* const room, const gchar* const msg)
 {
     if (prefs_get_boolean(PREF_GRLOG)) {
-        auto_char char* mybarejid = connection_get_barejid();
-        char* mynick = muc_nick(room);
-        _groupchat_log_chat(mybarejid, room, mynick, msg);
+        const char* const mynick = muc_nick(room);
+        _groupchat_log_chat(connection_get_barejid(), room, mynick, msg);
     }
 }
 
@@ -318,8 +321,7 @@ void
 groupchat_log_msg_in(const gchar* const room, const gchar* const nick, const gchar* const msg)
 {
     if (prefs_get_boolean(PREF_GRLOG)) {
-        auto_char char* mybarejid = connection_get_barejid();
-        _groupchat_log_chat(mybarejid, room, nick, msg);
+        _groupchat_log_chat(connection_get_barejid(), room, nick, msg);
     }
 }
 
@@ -327,9 +329,9 @@ void
 groupchat_log_omemo_msg_out(const gchar* const room, const gchar* const msg)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        auto_char char* mybarejid = connection_get_barejid();
+        const char* mybarejid = connection_get_barejid();
         auto_gchar gchar* pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
-        char* mynick = muc_nick(room);
+        const char* const mynick = muc_nick(room);
 
         if (strcmp(pref_omemo_log, "on") == 0) {
             _groupchat_log_chat(mybarejid, room, mynick, msg);
@@ -343,7 +345,7 @@ void
 groupchat_log_omemo_msg_in(const gchar* const room, const gchar* const nick, const gchar* const msg)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        auto_char char* mybarejid = connection_get_barejid();
+        const char* mybarejid = connection_get_barejid();
         auto_gchar gchar* pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
 
         if (strcmp(pref_omemo_log, "on") == 0) {
@@ -394,13 +396,6 @@ _groupchat_log_chat(const gchar* const login, const gchar* const room, const gch
     g_date_time_unref(dt_tmp);
 }
 
-void
-chat_log_close(void)
-{
-    g_hash_table_destroy(logs);
-    g_hash_table_destroy(groupchat_logs);
-}
-
 static char*
 _get_log_filename(const char* const other, const char* const login, GDateTime* dt, gboolean is_room)
 {
