@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2019 Aurelien Aptel <aurelien.aptel@gmail.com>
- * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2026 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -430,13 +430,18 @@ out:
 }
 
 void
-color_pair_cache_reset(void)
+color_pair_cache_free(void)
 {
     if (cache.pairs) {
         free(cache.pairs);
         memset(&cache, 0, sizeof(cache));
     }
+}
 
+void
+color_pair_cache_reset(void)
+{
+    color_pair_cache_free();
     /*
      * COLOR_PAIRS is actually not a macro and is thus not a
      * compile-time constant
