@@ -81,6 +81,7 @@
 static int inp_size;
 static gboolean perform_resize = FALSE;
 static GTimer* ui_idle_time;
+static WINDOW* main_scr;
 
 #ifdef HAVE_LIBXSS
 static Display* display;
@@ -88,11 +89,28 @@ static Display* display;
 
 static void _ui_draw_term_title(void);
 
+static void
+_ui_close(void)
+{
+    g_timer_destroy(ui_idle_time);
+    notifier_uninit();
+    cons_clear_alerts();
+    wins_destroy();
+    inp_close();
+    status_bar_close();
+    free_title_bar();
+    endwin();
+    delwin(main_scr);
+    main_scr = NULL;
+    delscreen(set_term(NULL));
+}
+
 void
 ui_init(void)
 {
     log_info("Initialising UI");
-    initscr();
+    prof_add_shutdown_routine(_ui_close);
+    main_scr = initscr();
     nonl();
     cbreak();
     noecho();
@@ -107,7 +125,11 @@ ui_init(void)
     notifier_initialise();
     cons_about();
 #ifdef HAVE_LIBXSS
-    display = XOpenDisplay(0);
+    char* x11_display = getenv("DISPLAY");
+    char* wayland_display = getenv("WAYLAND_DISPLAY");
+    if (x11_display && !wayland_display) {
+        display = XOpenDisplay(0);
+    }
 #endif
     ui_idle_time = g_timer_new();
     inp_size = 0;
@@ -150,15 +172,17 @@ ui_get_idle_time(void)
 {
 // if compiled with libxss, get the x sessions idle time
 #ifdef HAVE_LIBXSS
-    XScreenSaverInfo* info = XScreenSaverAllocInfo();
-    if (info && display) {
-        XScreenSaverQueryInfo(display, DefaultRootWindow(display), info);
-        unsigned long result = info->idle;
-        XFree(info);
-        return result;
-    }
-    if (info) {
-        XFree(info);
+    if (display) {
+        XScreenSaverInfo* info = XScreenSaverAllocInfo();
+        if (info) {
+            XScreenSaverQueryInfo(display, DefaultRootWindow(display), info);
+            unsigned long result = info->idle;
+            XFree(info);
+            return result;
+        }
+        if (info) {
+            XFree(info);
+        }
     }
 // if no libxss or xss idle time failed, use profanity idle time
 #endif
@@ -173,17 +197,6 @@ ui_reset_idle_time(void)
     g_timer_start(ui_idle_time);
 }
 
-void
-ui_close(void)
-{
-    notifier_uninit();
-    cons_clear_alerts();
-    wins_destroy();
-    inp_close();
-    status_bar_close();
-    endwin();
-}
-
 void
 ui_resize(void)
 {
@@ -293,18 +306,7 @@ ui_contact_typing(const char* const barejid, const char* const resource)
             is_current = wins_is_current(window);
         }
         if (!is_current || (is_current && prefs_get_boolean(PREF_NOTIFY_TYPING_CURRENT))) {
-            PContact contact = roster_get_contact(barejid);
-            char const* display_usr = NULL;
-            if (contact) {
-                if (p_contact_name(contact)) {
-                    display_usr = p_contact_name(contact);
-                } else {
-                    display_usr = barejid;
-                }
-            } else {
-                display_usr = barejid;
-            }
-            notify_typing(display_usr);
+            notify_typing(roster_get_display_name(barejid));
         }
     }
 }
@@ -359,7 +361,7 @@ void
 ui_handle_login_account_success(ProfAccount* account, gboolean secured)
 {
     if (account->theme) {
-        if (theme_load(account->theme, false)) {
+        if (theme_load(account->theme, FALSE)) {
             ui_load_colours();
             if (prefs_get_boolean(PREF_ROSTER)) {
                 ui_show_roster();
@@ -706,6 +708,8 @@ ui_close_win(int index)
         }
     }
 
+    // remove the IQ handlers
+    iq_handlers_remove_win(window);
     wins_close_by_num(index);
     title_bar_console();
     status_bar_current(1);
@@ -781,7 +785,7 @@ ui_room_join(const char* const roomjid, gboolean focus)
     }
     ProfWin* window = (ProfWin*)mucwin;
 
-    char* nick = muc_nick(roomjid);
+    const char* const nick = muc_nick(roomjid);
     win_print(window, THEME_ROOMINFO, "!", "-> You have joined the room as %s", nick);
     if (prefs_get_boolean(PREF_MUC_PRIVILEGES)) {
         char* role = muc_role_str(roomjid);
