@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2026 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -98,6 +98,18 @@ static xmpp_stanza_t* _ox_openpgp_signcrypt(xmpp_ctx_t* ctx, const char* const t
 
 static GHashTable* pubsub_event_handlers;
 
+gchar*
+get_display_name(const ProfMessage* const message, int* flags)
+{
+    if (equals_our_barejid(message->from_jid->barejid)) {
+        return g_strdup("me");
+    } else {
+        if (flags)
+            *flags = NO_ME;
+        return roster_get_msg_display_name(message->from_jid->barejid, message->from_jid->resourcepart);
+    }
+}
+
 static gboolean
 _handled_by_plugin(xmpp_stanza_t* const stanza)
 {
@@ -118,7 +130,10 @@ _handle_headline(xmpp_stanza_t* const stanza)
     if (body) {
         char* text = xmpp_stanza_get_text(body);
         if (text) {
-            cons_show("Headline: %s", text);
+            const char* const stanza_from = xmpp_stanza_get_from(stanza);
+            if (stanza_from) {
+                cons_show("Headline from %s: %s", stanza_from, text);
+            }
             xmpp_free(connection_get_ctx(), text);
         }
     }
@@ -150,6 +165,7 @@ static int
 _message_handler(xmpp_conn_t* const conn, xmpp_stanza_t* const stanza, void* const userdata)
 {
     log_debug("Message stanza handler fired");
+    autoping_timer_extend();
 
     if (_handled_by_plugin(stanza)) {
         return 1;
@@ -243,11 +259,10 @@ _message_handler(xmpp_conn_t* const conn, xmpp_stanza_t* const stanza, void* con
         if (carbons) {
 
             // carbon must come from ourselves
-            auto_char char* mybarejid = connection_get_barejid();
             const char* const stanza_from = xmpp_stanza_get_from(stanza);
 
             if (stanza_from) {
-                if (g_strcmp0(mybarejid, stanza_from) != 0) {
+                if (!equals_our_barejid(stanza_from)) {
                     log_warning("Invalid carbon received, from: %s", stanza_from);
                     msg_stanza = NULL;
                 } else {
@@ -305,13 +320,9 @@ _handle_form(xmpp_stanza_t* const stanza)
     return TRUE;
 }
 
-void
-message_handlers_init(void)
+static void
+_message_handlers_cleanup(void)
 {
-    xmpp_conn_t* const conn = connection_get_conn();
-    xmpp_ctx_t* const ctx = connection_get_ctx();
-    xmpp_handler_add(conn, _message_handler, NULL, STANZA_NAME_MESSAGE, NULL, ctx);
-
     if (pubsub_event_handlers) {
         GList* keys = g_hash_table_get_keys(pubsub_event_handlers);
         GList* curr = keys;
@@ -325,26 +336,26 @@ message_handlers_init(void)
         g_list_free(keys);
         g_hash_table_destroy(pubsub_event_handlers);
     }
+    pubsub_event_handlers = NULL;
+}
 
+void
+message_handlers_init(void)
+{
+    prof_add_shutdown_routine(_message_handlers_cleanup);
+    xmpp_conn_t* const conn = connection_get_conn();
+    xmpp_ctx_t* const ctx = connection_get_ctx();
+    xmpp_handler_add(conn, _message_handler, NULL, STANZA_NAME_MESSAGE, NULL, ctx);
+    _message_handlers_cleanup();
     pubsub_event_handlers = g_hash_table_new_full(g_str_hash, g_str_equal, free, free);
 }
 
 ProfMessage*
 message_init(void)
 {
-    ProfMessage* message = malloc(sizeof(ProfMessage));
+    ProfMessage* message = calloc(1, sizeof(ProfMessage));
 
-    message->from_jid = NULL;
-    message->to_jid = NULL;
-    message->id = NULL;
-    message->originid = NULL;
-    message->stanzaid = NULL;
-    message->replace_id = NULL;
-    message->body = NULL;
-    message->encrypted = NULL;
-    message->plain = NULL;
     message->enc = PROF_MSG_ENC_NONE;
-    message->timestamp = NULL;
     message->trusted = true;
     message->type = PROF_MSG_TYPE_UNINITIALIZED;
 
@@ -462,8 +473,7 @@ message_send_chat_pgp(const char* const barejid, const char* const msg, gboolean
 
     xmpp_stanza_t* message = NULL;
 #ifdef HAVE_LIBGPGME
-    char* account_name = session_get_account_name();
-    ProfAccount* account = accounts_get_account(account_name);
+    ProfAccount* account = accounts_get_account(session_get_account_name());
     if (account->pgp_keyid) {
         auto_jid Jid* jidp = jid_create(jid);
         auto_char char* encrypted = p_gpg_encrypt(jidp->barejid, msg, account->pgp_keyid);
@@ -525,8 +535,7 @@ message_send_chat_ox(const char* const barejid, const char* const msg, gboolean
 
     xmpp_stanza_t* message = NULL;
 
-    char* account_name = session_get_account_name();
-    ProfAccount* account = accounts_get_account(account_name);
+    ProfAccount* account = accounts_get_account(session_get_account_name());
 
     message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, jid, id);
     xmpp_message_set_body(message, "This message is encrypted (XEP-0373: OpenPGP for XMPP).");
@@ -991,13 +1000,7 @@ _room_config_handler(xmpp_stanza_t* const stanza, void* const userdata)
     xmpp_stanza_t* query = xmpp_stanza_get_child_by_name(stanza, STANZA_NAME_QUERY);
     EntityCapabilities* capabilities = stanza_create_caps_from_query_element(query);
 
-    // Update window name
-    ProfMucWin* win = wins_get_muc(from);
-    if (win != NULL) {
-        free(win->room_name);
-        win->room_name = strdup(capabilities->identity->name);
-
-        // Update features
+    if (mucwin_set_room_name(from, capabilities->identity->name)) {
         muc_set_features(from, capabilities->features);
     }
     caps_destroy(capabilities);
@@ -1103,14 +1106,6 @@ _handle_groupchat(xmpp_stanza_t* const stanza)
         }
     }
 
-    xmpp_stanza_t* replace_id_stanza = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_LAST_MESSAGE_CORRECTION);
-    if (replace_id_stanza) {
-        const char* replace_id = xmpp_stanza_get_id(replace_id_stanza);
-        if (replace_id) {
-            message->replace_id = strdup(replace_id);
-        }
-    }
-
     message->body = xmpp_message_get_body(stanza);
 
     // check omemo encryption
@@ -1155,6 +1150,14 @@ _handle_groupchat(xmpp_stanza_t* const stanza)
     if (is_muc_history) {
         sv_ev_room_history(message);
     } else {
+        // XEP-0308 states: `corrections must not be allowed (by the receiver) for messages received before the sender joined the room`
+        xmpp_stanza_t* replace_id_stanza = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_LAST_MESSAGE_CORRECTION);
+        if (replace_id_stanza) {
+            const char* replace_id = xmpp_stanza_get_id(replace_id_stanza);
+            if (replace_id) {
+                message->replace_id = strdup(replace_id);
+            }
+        }
         sv_ev_room_message(message);
     }
 
@@ -1249,7 +1252,7 @@ _handle_muc_private_message(xmpp_stanza_t* const stanza)
     ProfMessage* message = message_init();
     message->type = PROF_MSG_TYPE_MUCPM;
 
-    const gchar* from = xmpp_stanza_get_from(stanza);
+    const char* from = xmpp_stanza_get_from(stanza);
     if (!from) {
         goto out;
     }
@@ -1348,7 +1351,7 @@ _handle_chat(xmpp_stanza_t* const stanza, gboolean is_mam, gboolean is_carbon, c
         return;
     }
 
-    const gchar* from = xmpp_stanza_get_from(stanza);
+    const char* from = xmpp_stanza_get_from(stanza);
     if (!from) {
         return;
     }
@@ -1452,10 +1455,8 @@ _handle_chat(xmpp_stanza_t* const stanza, gboolean is_mam, gboolean is_carbon, c
 
     if (message->plain || message->body || message->encrypted) {
         if (is_carbon) {
-            auto_char char* mybarejid = connection_get_barejid();
-
             // if we are the recipient, treat as standard incoming message
-            if (g_strcmp0(mybarejid, message->to_jid->barejid) == 0) {
+            if (equals_our_barejid(message->to_jid->barejid)) {
                 sv_ev_incoming_carbon(message);
                 // else treat as a sent message
             } else {
@@ -1574,7 +1575,7 @@ _send_message_stanza(xmpp_stanza_t* const stanza)
     if (plugin_text) {
         xmpp_send_raw_string(conn, "%s", plugin_text);
     } else {
-        xmpp_send_raw_string(conn, "%s", text);
+        xmpp_send_raw(conn, text, text_size);
     }
     xmpp_free(connection_get_ctx(), text);
 }
