@@ -87,9 +87,8 @@ omemo_devicelist_configure(void)
 {
     xmpp_ctx_t* const ctx = connection_get_ctx();
     auto_char char* id = connection_create_stanza_id();
-    auto_jid Jid* jid = jid_create(connection_get_fulljid());
 
-    xmpp_stanza_t* iq = stanza_create_pubsub_configure_request(ctx, id, jid->barejid, STANZA_NS_OMEMO_DEVICELIST);
+    xmpp_stanza_t* iq = stanza_create_pubsub_configure_request(ctx, id, connection_get_jid()->barejid, STANZA_NS_OMEMO_DEVICELIST);
 
     iq_id_handler_add(id, _omemo_devicelist_configure_submit, NULL, NULL);
 
@@ -117,6 +116,11 @@ omemo_devicelist_request(const char* const jid)
 void
 omemo_bundle_publish(gboolean first)
 {
+    if (!connection_supports(XMPP_FEATURE_PUBSUB_PUBLISH_OPTIONS)) {
+        cons_show("OMEMO: Cannot publish bundle: no PUBSUB feature announced");
+        log_debug("[OMEMO] Cannot publish bundle: no PUBSUB feature announced");
+        return;
+    }
     log_debug("[OMEMO] publish own OMEMO bundle");
     xmpp_ctx_t* const ctx = connection_get_ctx();
     unsigned char* identity_key = NULL;
@@ -142,14 +146,10 @@ omemo_bundle_publish(gboolean first)
     g_list_free(lengths);
     g_list_free(ids);
 
-    if (connection_supports(XMPP_FEATURE_PUBSUB_PUBLISH_OPTIONS)) {
-        stanza_attach_publish_options_va(ctx, iq,
-                                         4, // 2 * number of key-value pairs
-                                         "pubsub#persist_items", "true",
-                                         "pubsub#access_model", "open");
-    } else {
-        log_debug("[OMEMO] Cannot publish bundle: no PUBSUB feature announced");
-    }
+    stanza_attach_publish_options_va(ctx, iq,
+                                     4, // 2 * number of key-value pairs
+                                     "pubsub#persist_items", "true",
+                                     "pubsub#access_model", "open");
 
     iq_id_handler_add(id, _omemo_bundle_publish_result, NULL, GINT_TO_POINTER(first));
 
@@ -196,9 +196,7 @@ omemo_start_device_session_handle_bundle(xmpp_stanza_t* const stanza, void* cons
     }
 
     if (!from_attr) {
-        Jid* jid = jid_create(connection_get_fulljid());
-        from = strdup(jid->barejid);
-        jid_destroy(jid);
+        from = strdup(connection_get_jid()->barejid);
     } else {
         from = strdup(from_attr);
     }
@@ -394,16 +392,16 @@ omemo_receive_message(xmpp_stanza_t* const stanza, gboolean* trusted)
 
     xmpp_stanza_t* payload = xmpp_stanza_get_child_by_name(encrypted, "payload");
     if (!payload) {
-        return NULL;
+        goto quit;
     }
     payload_text = xmpp_stanza_get_text(payload);
     if (!payload_text) {
-        return NULL;
+        goto quit;
     }
     size_t payload_len;
     payload_raw = g_base64_decode(payload_text, &payload_len);
     if (!payload_raw) {
-        return NULL;
+        goto quit;
     }
 
     xmpp_stanza_t* key_stanza;
@@ -447,6 +445,7 @@ omemo_receive_message(xmpp_stanza_t* const stanza, gboolean* trusted)
         g_list_free_full(keys, (GDestroyNotify)omemo_key_free);
     }
 
+quit:
     g_free(iv_raw);
     g_free(payload_raw);
     g_free(iv_text);
@@ -608,9 +607,8 @@ _omemo_devicelist_configure_submit(xmpp_stanza_t* const stanza, void* const user
     form_set_value(form, "pubsub#access_model", "open");
 
     xmpp_ctx_t* const ctx = connection_get_ctx();
-    auto_jid Jid* jid = jid_create(connection_get_fulljid());
     auto_char char* id = connection_create_stanza_id();
-    xmpp_stanza_t* iq = stanza_create_pubsub_configure_submit(ctx, id, jid->barejid, STANZA_NS_OMEMO_DEVICELIST, form);
+    xmpp_stanza_t* iq = stanza_create_pubsub_configure_submit(ctx, id, connection_get_jid()->barejid, STANZA_NS_OMEMO_DEVICELIST, form);
 
     iq_id_handler_add(id, _omemo_devicelist_configure_result, NULL, NULL);
 
@@ -633,10 +631,7 @@ _omemo_devicelist_configure_result(xmpp_stanza_t* const stanza, void* const user
     log_debug("[OMEMO] node configured");
 
     // Try to publish
-    char* barejid = connection_get_barejid();
-    omemo_devicelist_request(barejid);
-
-    free(barejid);
+    omemo_devicelist_request(connection_get_barejid());
 
     return 0;
 }
@@ -660,12 +655,11 @@ _omemo_bundle_publish_result(xmpp_stanza_t* const stanza, void* const userdata)
 
     log_debug("[OMEMO] cannot publish bundle with open access model, trying to configure node");
     xmpp_ctx_t* const ctx = connection_get_ctx();
-    auto_jid Jid* jid = jid_create(connection_get_fulljid());
     auto_char char* id = connection_create_stanza_id();
     auto_gchar gchar* node = g_strdup_printf("%s:%d", STANZA_NS_OMEMO_BUNDLES, omemo_device_id());
     log_debug("[OMEMO] node: %s", node);
 
-    xmpp_stanza_t* iq = stanza_create_pubsub_configure_request(ctx, id, jid->barejid, node);
+    xmpp_stanza_t* iq = stanza_create_pubsub_configure_request(ctx, id, connection_get_jid()->barejid, node);
 
     iq_id_handler_add(id, _omemo_bundle_publish_configure, NULL, userdata);
 
@@ -700,10 +694,9 @@ _omemo_bundle_publish_configure(xmpp_stanza_t* const stanza, void* const userdat
     form_set_value(form, "pubsub#access_model", "open");
 
     xmpp_ctx_t* const ctx = connection_get_ctx();
-    auto_jid Jid* jid = jid_create(connection_get_fulljid());
     auto_char char* id = connection_create_stanza_id();
     auto_gchar gchar* node = g_strdup_printf("%s:%d", STANZA_NS_OMEMO_BUNDLES, omemo_device_id());
-    xmpp_stanza_t* iq = stanza_create_pubsub_configure_submit(ctx, id, jid->barejid, node, form);
+    xmpp_stanza_t* iq = stanza_create_pubsub_configure_submit(ctx, id, connection_get_jid()->barejid, node, form);
 
     iq_id_handler_add(id, _omemo_bundle_publish_configure_result, NULL, userdata);
 
